package ca.yorku.cse.mack.tilttarget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.media.MediaPlayer;
import android.os.CountDownTimer;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.view.View;

/**
 * TiltTarget
 *
 * @author Scott MacKenzie
 */
public class ExperimentPanel extends View
{
    final String MYDEBUG = "MYDEBUG"; // for Log.i messages

    final float DEGREES_TO_RADIANS = 0.0174532925f;
    final int GUTTER_WIDTH = 16; // density-independent pixels (also sets the ball width)
    final int RESULTS_TEXT_SIZE = 20; // may have to tweak, depending on device
    final String KEY_LABELS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    final String FIVE_BY_THREE = "5x3";
    final String FOUR_BY_TEN = "4x10";
    int ballDiameter;
    int gutterWidth;
    int xOffset, yOffset; // for drawing results
    float resultsTextSize;
    String keyboard, orderOfControl;
    String[] keyLabel;
    int[] keyIndex;
    float velocity;
    int keyIndexToSelect;
    float dT;
    long now, lastT;

    Vibrator vib;
    MediaPlayer clickSound;
    Bitmap ball;
    Key[] key;
    Key tentativeKey, greenContinueKey;
    int numberOfKeys, numberOfRows, numberOfColumns;
    float keyHeight, keyWidth;
    int panelWidth, panelHeight;
    boolean beginSequence; // set when green key selected, cleared first time doNewSequence executes
    float xBall, yBall; // top-left of the ball (for painting)
    float xBallCenter, yBallCenter; // Centre of the ball
    boolean vibrotactileFeedback, auditoryFeedback;
    long dwellTime;
    boolean dwellPending;
    DwellTimer dwellTimer;
    float tiltAngle, tiltMagnitude;
    int tiltGain;
    float dBall; // the amount to move the ball (in pixels): dBall = dT * velocity
    float pixelDensity;
    float xCenter, yCenter; // the centre of the screen
    Paint infoBackgroundPaint, resultsTextPaint, normalPaint, fillPaint, fillHoverPaint, greenFillPaint, textPaint;

    // NOTE: these variables are accessed in TiltTargetActivity
    boolean keySelected = false; // any key
    boolean targetKeySelected = false; // the key to select
    boolean resultsMode = true; // don't allow input until enabled in main activity
    StringBuilder infoString;
    boolean done = false; // ...but give the user a chance to view results from last phrase
    boolean reallyDone = false; // close files and exit
    //-----

    boolean readyToDraw = false;

    public ExperimentPanel(Context ctx)
    {
        super(ctx);
        initialize(ctx);
    }

    public ExperimentPanel(Context ctx, AttributeSet attrs)
    {
        super(ctx, attrs);
        initialize(ctx);
    }

    public ExperimentPanel(Context ctx, AttributeSet attrs, int defStyle)
    {
        super(ctx, attrs, defStyle);
        initialize(ctx);
    }

    // things that can be initialised from within this View
    private void initialize(Context c)
    {
        // get display pixel density (and init things that depend on pixelDensity)
        pixelDensity = c.getResources().getDisplayMetrics().density;
        gutterWidth = (int)(GUTTER_WIDTH * pixelDensity);
        resultsTextSize = RESULTS_TEXT_SIZE * pixelDensity;
        xOffset = gutterWidth + (int)(10 * pixelDensity);
        yOffset = gutterWidth + (int)(10 * pixelDensity + resultsTextSize);

        // adjust ball diameter to be 25% of key width or height (whichever is less)
        ballDiameter = gutterWidth;
        Bitmap temp = BitmapFactory.decodeResource(getResources(), R.drawable.ball);
        ball = Bitmap.createScaledBitmap(temp, ballDiameter, ballDiameter, true); // (re)adjust ball diameter in bitmap

        // get a vibrator (only used of vibrate setting checked)
        vib = (Vibrator)c.getSystemService(Context.VIBRATOR_SERVICE);

        infoBackgroundPaint = new Paint();
        infoBackgroundPaint.setColor(Color.BLACK);
        infoBackgroundPaint.setStyle(Paint.Style.FILL);

        resultsTextPaint = new Paint();
        resultsTextPaint.setColor(Color.WHITE);
        resultsTextPaint.setAntiAlias(true);
        resultsTextPaint.setTextSize(resultsTextSize);

        infoString = new StringBuilder(500);
        infoString.append("WELCOME\n \nTo begin...\nPosition ball in\nGREEN square");

        greenFillPaint = new Paint();
        greenFillPaint.setColor(Color.GREEN);
        greenFillPaint.setStyle(Paint.Style.FILL);

        normalPaint = new Paint();
        normalPaint.setColor(0xffff9999); // lighter red (to minimize distraction)
        normalPaint.setStyle(Paint.Style.STROKE);
        normalPaint.setStrokeWidth(2);
        normalPaint.setAntiAlias(true);

        fillPaint = new Paint();
        fillPaint.setColor(0xffff9999); // lighter red (to minimize distraction)
        fillPaint.setStyle(Paint.Style.FILL);
        fillPaint.setStrokeWidth(2);
        fillPaint.setAntiAlias(true);

        fillHoverPaint = new Paint();
        fillHoverPaint.setColor(0xffff6666); // slight more red (to indicate ball-over)
        fillHoverPaint.setStyle(Paint.Style.FILL);
        fillHoverPaint.setStrokeWidth(2);
        fillHoverPaint.setAntiAlias(true);

        textPaint = new Paint();
        textPaint.setColor(Color.BLUE);
        textPaint.setAntiAlias(true);
        textPaint.setTextAlign(Paint.Align.CENTER);

        beginSequence = true;

        this.setBackgroundColor(Color.LTGRAY);
    }

    /**
     * Here's where we get the width and height of the experiment panel.  Initialize things that depends on these
     * parameters.
     */
    @Override
    public void onWindowFocusChanged(boolean hasFocus)
    {
        if (!hasFocus)
            return;

        panelWidth = this.getWidth();
        panelHeight = this.getHeight();

        keyWidth = (panelWidth - 2 * gutterWidth) / numberOfColumns;
        keyHeight = (panelHeight - 2 * gutterWidth) / numberOfRows;

        // need keyHeight to specify text height for keys
        textPaint.setTextSize(0.6f * keyHeight);

        for (int i = 0; i < key.length; ++i)
        {
            float x = gutterWidth + (i % numberOfColumns) * keyWidth;
            float y = gutterWidth + (i / numberOfColumns) * keyHeight;
            key[i].setIdx(i);
            key[i].setCoordinates(x, y, keyWidth, keyHeight);
        }
        greenContinueKey = new Key("", -1);
        greenContinueKey.setIdx(0);
        greenContinueKey.setCoordinates(panelWidth - ballDiameter, 0, ballDiameter + 1, ballDiameter + 1);

        xCenter = panelWidth / 2f;
        yCenter = panelHeight / 2f;
        xBall = xCenter; // start the ball in the centre of the display
        yBall = yCenter;

        readyToDraw = true;
    }

    // things to initialise from the Activity
    public void setParameters(String keyboardArg, String orderOfControlArg,
                              int tiltGainArg, int dwellTimeArg,
                              boolean vibrotactileFeedbackArg,
                              MediaPlayer clickSoundArg, boolean auditoryFeedbackArg)
    {
        keyboard = keyboardArg;
        orderOfControl = orderOfControlArg;
        tiltGain = tiltGainArg;
        dwellTime = dwellTimeArg;
        vibrotactileFeedback = vibrotactileFeedbackArg;
        clickSound = clickSoundArg;
        auditoryFeedback = auditoryFeedbackArg;

        if (keyboard.equals(FIVE_BY_THREE)) // phone-like
        {
            numberOfRows = 5;
            numberOfColumns = 3;
            numberOfKeys = 15;
        } else if (keyboard.equals(FOUR_BY_TEN)) // qwerty-like
        {
            numberOfRows = 4;
            numberOfColumns = 10;
            numberOfKeys = 40;
        }
        key = new Key[numberOfKeys];
        keyLabel = new String[numberOfKeys];
        keyIndex = new int[numberOfKeys];

        for (int i = 0; i < numberOfKeys; ++i)
        {
            keyLabel[i] = "" + KEY_LABELS.charAt(i);
            keyIndex[i] = i;
            key[i] = new Key(keyLabel[i], i);
        }

        // initialize the dwell timer, giving it the dwell time from the setup up dialog
        dwellTimer = new DwellTimer(dwellTime, dwellTime);

    } // end setParameters

    // receive the sensor data from the main activity and use the data to compute ball position, etc.
    public void updateTilt(float tiltAngleArg, float tiltMagnitudeArg)
    {
        tiltAngle = tiltAngleArg;
        tiltMagnitude = tiltMagnitudeArg;

        // don't allow tiltMagnitude to exceed 45 degrees
        final float MAX_MAGNITUDE = 45f;
        tiltMagnitude = tiltMagnitude > MAX_MAGNITUDE ? MAX_MAGNITUDE : tiltMagnitude;

        // get current time and delta since last onDraw
        now = System.nanoTime();
        dT = (now - lastT) / 1000000000f; // seconds
        lastT = now;

        // This is the only code that distinguishes velocity-control from position-control
        if (orderOfControl.equals("Velocity")) // velocity control
        {
            // compute how far the ball should move
            velocity = tiltMagnitude * tiltGain;
            dBall = dT * velocity; // make the ball move this amount (pixels)

            // compute the ball's new coordinates
            float dx = (float)Math.sin(tiltAngle * DEGREES_TO_RADIANS) * dBall;
            float dy = -(float)Math.cos(tiltAngle * DEGREES_TO_RADIANS) * dBall;
            xBall += dx;
            yBall += dy;
        } else // position control
        {
            // compute how far the ball should move
            dBall = tiltMagnitude * tiltGain;

            // compute the ball's new coordinates
            float dx = (float)Math.sin(tiltAngle * DEGREES_TO_RADIANS) * dBall;
            float dy = -(float)Math.cos(tiltAngle * DEGREES_TO_RADIANS) * dBall;
            xBall = xCenter + dx;
            yBall = yCenter + dy;
        }

        // keep the ball visible
        if (xBall < 0) xBall = 0;
        else if (xBall > panelWidth - ballDiameter) xBall = panelWidth - ballDiameter;
        if (yBall < 0) yBall = 0;
        else if (yBall > panelHeight - ballDiameter) yBall = panelHeight - ballDiameter;

        // adjust the ball's centre coordinates
        xBallCenter = xBall + ballDiameter / 2f;
        yBallCenter = yBall + ballDiameter / 2f;

        if (resultsMode) // look for selection in the green circle
        {
            // if the ball enters the green key, start the dwell timer (unless we're done)
            if (greenContinueKey.inKey(xBallCenter, yBallCenter, ballDiameter))
            {
                if (!dwellPending && !reallyDone)
                {
                    dwellPending = true;
                    dwellTimer.start();
                }
            }

            // cancel the dwell timer if the ball exits the green key before the dwell time elapses
            if (dwellPending && !greenContinueKey.inKey(xBallCenter, yBallCenter, ballDiameter))
            {
                dwellPending = false;
                dwellTimer.cancel();
            }

        } else if (key != null) // selection mode (looking for target selections)
        {
            // determine which key, if any, the ball is inside
            for (Key k : key)
            {
                if (k.inKey(xBallCenter, yBallCenter, ballDiameter))
                {
                    // this is (tentatively) the key the user is trying to select
                    // (NOTE: dwell time must elapse before selection)
                    tentativeKey = k;

                    // start the dwell timer
                    if (!dwellPending && !resultsMode)
                    {
                        dwellPending = true;
                        dwellTimer.start();
                    }
                    break;
                }
            }

            // cancel the dwell timer if the ball exits the target key (before the dwell time elapses)
            if (dwellPending && !key[tentativeKey.index].inKey(xBallCenter, yBallCenter, ballDiameter))
            {
                dwellPending = false;
                dwellTimer.cancel();
            }
        }
        this.invalidate();
    }

    @Override
    protected void onDraw(Canvas canvas)
    {
        super.onDraw(canvas);

        if (!readyToDraw)
            return;

        // take one of two draw paths, depending on whether we are "between sequences" (resultsMode) or "within
        // sequences" (!resultsMode)
        if (resultsMode)
        {
            // ======================================================================================
            // draw the info text (start instructions or results) and the green key to start sequence
            // ======================================================================================

            // draw the rectangle within which the instructions/results are presented
            canvas.drawRect(gutterWidth, gutterWidth, panelWidth - gutterWidth, panelHeight - gutterWidth,
                    infoBackgroundPaint);

            String[] s = infoString.toString().split("\n");
            for (int i = 0, j = yOffset; i < s.length; ++i)
            {
                canvas.drawText(s[i], xOffset, j, resultsTextPaint);
                j += (int)(resultsTextSize * 1.25f);
            }

            // draw the green continue key
            canvas.drawRect(greenContinueKey.x, greenContinueKey.y, greenContinueKey.x + greenContinueKey.width, greenContinueKey.y +
                    greenContinueKey.height, greenFillPaint);

        } else
        {
            // =====================================================
            // draw the targets (with special shading for target key
            // =====================================================

            // iterate through the keys
            for (int i = 0; i < key.length; ++i)
            {
                // draw the keys
                canvas.drawRect(key[i].x, key[i].y, key[i].x + key[i].width, key[i].y + key[i].height, normalPaint);

                // if it's the target key, use a different fill colour
                if (i == keyIndexToSelect)
                {
                    if (key[i].inKey(xBallCenter, yBallCenter, ballDiameter))
                        canvas.drawRect(key[i].x, key[i].y, key[i].x + key[i].width, key[i].y + key[i].height,
                                fillHoverPaint);
                    else
                        canvas.drawRect(key[i].x, key[i].y, key[i].x + key[i].width, key[i].y + key[i].height,
                                fillPaint);
                }

                // draw the key text
                canvas.drawText(key[i].label, key[i].r.centerX(), key[i].r.centerY() - textPaint.getFontMetrics()
                        .ascent / 2.0f, textPaint);
            }
        }

        // draw the ball in its new location
        canvas.drawBitmap(ball, xBall, yBall, null);

    } // end onDraw

    private void doKeySelected()
    {
        // set keySelected
        keySelected = true; // used in TiltTargetActivity

        // provide feedback (as per setup)
        if (vibrotactileFeedback) vib.vibrate(5);
        if (auditoryFeedback) clickSound.start();

        if (resultsMode) // green key selected
        {
            beginSequence = true; // will be cleared in main activity in doNewSequence
            resultsMode = false;
        } else
        {
            // if the ball is in the target key, set targetKeySelected as well
            if (tentativeKey.index == keyIndexToSelect)
                targetKeySelected = true; // used in TiltTargetActivity
        }

        if (done)
            reallyDone = true; // finished last sequence *and* green key selected after results were displayed

        dwellTimer.cancel();
    }

    // selecting a key requires the ball to dwell on the key for "dwell time" milliseconds
    public class DwellTimer extends CountDownTimer
    {
        DwellTimer(long millisInFuture, long countDownInterval)
        {
            super(millisInFuture, countDownInterval);
        }

        @Override
        public void onTick(long millisUntilFinished)
        {
        }

        @Override
        public void onFinish()
        {
            dwellPending = false;
            doKeySelected();
        }
    }
}
